@@ -291,6 +291,8 @@ create table foreign_link (
      noticesync int not null default 1 /* comment 'notice synchronisation, bit 1 = sync outgoing, bit 2 = sync incoming, bit 3 = filter local replies' */,
      friendsync int not null default 2 /* comment 'friend synchronisation, bit 1 = sync outgoing, bit 2 = sync incoming */, 
      profilesync int not null default 1 /* comment 'profile synchronization, bit 1 = sync outgoing, bit 2 = sync incoming' */,
+     last_noticesync timestamp default null /* comment 'last time notices were imported' */,
+     last_friendsync timestamp default null /* comment 'last time friends were imported' */,
      created timestamp not null default CURRENT_TIMESTAMP /* comment 'date this record was created' */,
      modified timestamp /* comment 'date this record was modified' */,
 
@@ -428,7 +430,7 @@ create index group_inbox_created_idx on group_inbox using btree(created);
 /* Textsearch stuff */
 
 create index textsearch_idx on profile using gist(textsearch);
-create index noticecontent_idx on notice using gist(to_tsvector('english',content));
+create index noticecontent_idx on notice using gist(to_tsvector('japanese',content));
 create trigger textsearchupdate before insert or update on profile for each row
-execute procedure tsvector_update_trigger(textsearch, 'pg_catalog.english', nickname, fullname, location, bio, homepage);
+execute procedure tsvector_update_trigger(textsearch, 'pg_catalog.japanese', nickname, fullname, location, bio, homepage);
 
